@@ -4,6 +4,8 @@ import argparse
 import time
 import numpy as np
 import datetime
+import shutil
+import warnings
 
 def simulate(l, b, beamwidth=0.6, v_min=-400, v_max=400, plot_file=''):
 	import requests
@@ -80,6 +82,7 @@ def simulate(l, b, beamwidth=0.6, v_min=-400, v_max=400, plot_file=''):
 	else:
 		# Display plot
 		plt.show()
+	plt.clf()
 
 def predict(lat, lon, height=0, source='', date='', plot_sun=True, plot_file=''):
 	from astropy.time import Time
@@ -152,7 +155,11 @@ def predict(lat, lon, height=0, source='', date='', plot_sun=True, plot_file='')
 	plt.xlim(0*u.hour, 24*u.hour)
 	plt.xticks((np.arange(24))*u.hour)
 	plt.ylim(0*u.deg, 90*u.deg)
-	plt.title(source+' | '+date, y=1.01)
+
+	if source != '':
+		plt.title(source+' | '+date, y=1.01)
+	else:
+		plt.title('Sun | '+date, y=1.01)
 
 	if utcoffset/u.hour != 0:
 		plt.xlabel('Time (UTC+'+str(int(utcoffset/u.hour))+')')
@@ -164,6 +171,7 @@ def predict(lat, lon, height=0, source='', date='', plot_sun=True, plot_file='')
 		plt.savefig(plot_file, bbox_inches='tight', pad_inches=0.2)
 	else:
 		plt.show()
+	plt.clf()
 
 def equatorial(alt, az, lat, lon, height=0):
     from astropy.time import Time
@@ -219,7 +227,7 @@ def observe(obs_parameters, spectrometer='wola', obs_file='observation.dat', sta
 		except:
 			from .run_wola import run_observation
 
-	dev_args = '"'+obs_parameters['dev_args']+'"'
+	dev_args = obs_parameters['dev_args']
 	rf_gain = obs_parameters['rf_gain']
 	if_gain = obs_parameters['if_gain']
 	bb_gain = obs_parameters['bb_gain']
@@ -271,11 +279,8 @@ channels='''+str(channels)+'''
 t_sample='''+str(t_sample)+'''
 duration='''+str(duration))
 
-### def observe_psr():
-### TO DO
-
-def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.dat',
-         cal_file='', waterfall_fits='', spectra_csv='', power_csv='', plot_file='plot.png'):
+def plot(obs_parameters='', n=0, m=0, f_rest=0, slope_correction=False, dB=False, rfi=[0,0], xlim=[0,0], ylim=[0,0],
+	 obs_file='observation.dat', cal_file='', waterfall_fits='', spectra_csv='', power_csv='', plot_file='plot.png'):
 	import matplotlib
 	matplotlib.use('Agg') # Try commenting this line if you run into display/rendering errors
 	import matplotlib.pyplot as plt
@@ -299,14 +304,14 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 		if mask.size == 0:
 			mask = np.zeros_like(spectrum)
 
-		noise = np.std((spectrum[2:]-spectrum[:-2])[mask[1:-1] == 0])/np.sqrt(2)
+		noise = np.nanstd((spectrum[2:]-spectrum[:-2])[mask[1:-1] == 0])/np.sqrt(2)
 		background = np.nanmean(spectrum[mask == 0])
 
 		return (spectrum-background)/noise
 
 	def best_fit(power):
 		'''Compute best Gaussian fit'''
-		avg = np.mean(power)
+		avg = np.nanmean(power)
 		var = np.var(power)
 
 		gaussian_fit_x = np.linspace(np.min(power),np.max(power),100)
@@ -340,6 +345,9 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 			elif 't_sample' in headers[i]:
 				t_sample = float(headers[i].strip().split('=')[1])
 
+	# Transform frequency axis limits to MHz
+	xlim = [x / 1e6 for x in xlim]
+
 	# Define Radial Velocity axis limits
 	left_velocity_edge = -299792.458*(bandwidth-2*frequency+2*f_rest)/(bandwidth-2*frequency)
 	right_velocity_edge = 299792.458*(-bandwidth-2*frequency+2*f_rest)/(bandwidth+2*frequency)
@@ -354,18 +362,37 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 	# Delete first 3 rows (potentially containing outlier samples)
 	waterfall = waterfall[3:, :]
 
+	# Mask RFI-contaminated channels
+	if rfi != [0,0]:
+		# Frequency to channel transformation
+		rfi_lo = channels*(rfi[0] - (frequency - bandwidth/2))/bandwidth
+		rfi_hi = channels*(rfi[1] - (frequency - bandwidth/2))/bandwidth
+
+		# Blank channels
+		for i in range(int(rfi_lo), int(rfi_hi)):
+			waterfall[:, i] = np.nan
+
 	if cal_file != '':
 		waterfall_cal = offset*np.fromfile(cal_file, dtype='float32').reshape(-1, channels)/bins
 
 		# Delete first 3 rows (potentially containing outlier samples)
 		waterfall_cal = waterfall_cal[3:, :]
 
-	# Compute average specta
-	avg_spectrum = decibel(np.mean(waterfall, axis=0))
-	if cal_file != '': avg_spectrum_cal = decibel(np.nanmean(waterfall_cal, axis=0))
+		# Mask RFI-contaminated channels
+		if rfi != [0,0]:
+			# Blank channels
+			for i in range(int(rfi_lo), int(rfi_hi)):
+				waterfall_cal[:, i] = np.nan
+
+	# Compute average spectra
+	with warnings.catch_warnings():
+		warnings.filterwarnings(action='ignore', message='Mean of empty slice')
+		avg_spectrum = decibel(np.nanmean(waterfall, axis=0))
+		if cal_file != '':
+			avg_spectrum_cal = decibel(np.nanmean(waterfall_cal, axis=0))
 
 	# Define array for Time Series plot
-	power = decibel(np.mean(waterfall, axis=1))
+	power = decibel(np.nanmean(waterfall, axis=1))
 
 	# Number of sub-integrations
 	subs = waterfall.shape[0]
@@ -391,13 +418,20 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 		else:
 			spectrum = avg_spectrum/avg_spectrum_cal
 
+		spectrum = SNR(spectrum, mask)
+		if slope_correction:
+			idx = np.isfinite(frequency) & np.isfinite(spectrum)
+			fit = np.polyfit(frequency[idx], spectrum[idx], 1)
+			ang_coeff = fit[0]
+			intercept = fit[1]
+			fit_eq = ang_coeff*frequency + intercept
+			spectrum = SNR(spectrum-fit_eq, mask)
+
 		# Mitigate RFI (Frequency Domain)
 		if n != 0:
 			spectrum_clean = SNR(spectrum.copy(), mask)
 			for i in range(0, int(channels)):
-				spectrum_clean[i] = np.median(spectrum_clean[i:i+n])
-
-		spectrum = SNR(spectrum, mask)
+				spectrum_clean[i] = np.nanmedian(spectrum_clean[i:i+n])
 
 		# Apply position offset for Spectral Line label
 		text_offset = 60
@@ -406,7 +440,7 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 	if m != 0:
 		power_clean = power.copy()
 		for i in range(0, int(subs)):
-			power_clean[i] = np.median(power_clean[i:i+m])
+			power_clean[i] = np.nanmedian(power_clean[i:i+m])
 
 	# Write Waterfall to file (FITS)
 	if waterfall_fits != '':
@@ -468,7 +502,10 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 	# Plot Average Spectrum
 	ax1 = fig.add_subplot(gs[0, 0])
 	ax1.plot(frequency, avg_spectrum)
-	ax1.set_xlim(np.min(frequency), np.max(frequency))
+	if xlim == [0,0]:
+		ax1.set_xlim(np.min(frequency), np.max(frequency))
+	else:
+		ax1.set_xlim(xlim[0], xlim[1])
 	ax1.ticklabel_format(useOffset=False)
 	ax1.set_xlabel('Frequency (MHz)')
 	if dB:
@@ -481,7 +518,7 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 		ax1.set_title('Average Spectrum')
 	ax1.grid()
 
-	if f_rest != 0:
+	if xlim == [0,0] and f_rest != 0:
 		# Add secondary axis for Radial Velocity
 		ax1_secondary = ax1.twiny()
 		ax1_secondary.set_xlabel('Radial Velocity (km/s)', labelpad=5)
@@ -498,7 +535,10 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 		if n != 0:
 			ax2.plot(frequency, spectrum_clean, color='orangered', label='Median (n = '+str(n)+')')
 			ax2.set_ylim()
-		ax2.set_xlim(np.min(frequency), np.max(frequency))
+		if xlim == [0,0]:
+			ax2.set_xlim(np.min(frequency), np.max(frequency))
+		else:
+			ax2.set_xlim(xlim[0], xlim[1])
 		ax2.ticklabel_format(useOffset=False)
 		ax2.set_xlabel('Frequency (MHz)')
 		ax2.set_ylabel('Signal-to-Noise Ratio (S/N)')
@@ -512,7 +552,7 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 			else:
 				ax2.legend(loc='upper left')
 
-		if f_rest != 0:
+		if xlim == [0,0] and f_rest != 0:
 			# Add secondary axis for Radial Velocity
 			ax2_secondary = ax2.twiny()
 			ax2_secondary.set_xlabel('Radial Velocity (km/s)', labelpad=5)
@@ -528,8 +568,17 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 		ax3 = fig.add_subplot(gs[0, 2])
 	else:
 		ax3 = fig.add_subplot(gs[0, 1])
+
 	ax3.imshow(decibel(waterfall), origin='lower', interpolation='None', aspect='auto',
-               extent=[np.min(frequency), np.max(frequency), np.min(t), np.max(t)])
+		   extent=[np.min(frequency), np.max(frequency), np.min(t), np.max(t)])
+	if xlim == [0,0] and ylim != [0,0]:
+		ax3.set_ylim(ylim[0], ylim[1])
+	elif xlim != [0,0] and ylim == [0,0]:
+		ax3.set_xlim(xlim[0], xlim[1])
+	elif xlim != [0,0] and ylim != [0,0]:
+		ax3.set_xlim(xlim[0], xlim[1])
+		ax3.set_ylim(ylim[0], ylim[1])
+
 	ax3.ticklabel_format(useOffset=False)
 	ax3.set_xlabel('Frequency (MHz)')
 	ax3.set_ylabel('Relative Time (s)')
@@ -547,7 +596,10 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 	if m != 0:
 		ax4.plot(t, power_clean, color='orangered', label='Median (n = '+str(m)+')')
 		ax4.set_ylim()
-	ax4.set_xlim(0, np.max(t))
+	if ylim == [0,0]:
+		ax4.set_xlim(0, np.max(t))
+	else:
+		ax4.set_xlim(ylim[0], ylim[1])
 	ax4.set_xlabel('Relative Time (s)')
 	if dB:
 		ax4.set_ylabel('Relative Power (dB)')
@@ -583,13 +635,137 @@ def plot(obs_parameters='', n=0, m=0, f_rest=0, dB=False, obs_file='observation.
 	# Save plots to file
 	plt.tight_layout()
 	plt.savefig(plot_file)
+	plt.clf()
+
+def plot_rfi(rfi_parameters, data='rfi_data', dB=True, plot_file='plot.png'):
+	import matplotlib
+	matplotlib.use('Agg') # Try commenting this line if you run into display/rendering errors
+	import matplotlib.pyplot as plt
+	from matplotlib.gridspec import GridSpec
+
+	plt.rcParams['legend.fontsize'] = 18
+	plt.rcParams['axes.labelsize'] = 40
+	plt.rcParams['axes.titlesize'] = 50
+	plt.rcParams['xtick.labelsize'] = 34
+	plt.rcParams['ytick.labelsize'] = 34
+
+	f_lo = rfi_parameters['f_lo']
+	bandwidth = rfi_parameters['bandwidth']
+	channels = rfi_parameters['channels']
+	t_sample = rfi_parameters['t_sample']
+	duration = rfi_parameters['duration']
+
+	def decibel(x):
+		if dB: return 10.0*np.log10(x)
+		return x
+
+	# Transform sampling time to number of bins
+	bins = int(t_sample*bandwidth/channels)
+
+	offset = 1
+	total = []
+
+	# Count number of .dat files
+	n = len([f for f in os.listdir(data) if f.endswith('.dat') and os.path.isfile(os.path.join(data, f))])
+
+	for i in range(int(n)):
+		# Load data
+		waterfall = offset*np.fromfile(data+'/'+str(i)+'.dat', dtype='float32').reshape(-1, channels)/bins
+
+		# Delete first 3 rows (potentially containing outlier samples)
+		waterfall = waterfall[3:, :]
+
+		total.append(waterfall)
+
+	# Merge dynamic spectra
+	combined = np.concatenate(total, axis=1)
+
+	# Compute average spectra
+	avg_spectrum = np.mean(combined, axis=0)
+
+	# Compute frequency axis
+	allfreq = []
+
+	for i in range(int(n)):
+		f_total = np.linspace((f_lo+bandwidth*i)-0.5*bandwidth, (f_lo+bandwidth*i)+0.5*bandwidth, channels, endpoint=False)*1e-6
+		allfreq.append(f_total)
+
+	f_total = np.concatenate(allfreq)
+
+	# Initialize plot
+	fig = plt.figure(figsize=(5*n,20.25))
+	gs = GridSpec(1,1)
+
+	# Plot merged spectra
+	ax = fig.add_subplot(gs[0,0])
+
+	ax.plot(f_total, decibel(avg_spectrum), '#3182bd')
+	ax.set_ylim()
+	ax.fill_between(f_total, decibel(avg_spectrum), y2=-1000, color='#deebf7')
+	ax.set_xlim(np.min(f_total), np.max(f_total))
+	ax.ticklabel_format(useOffset=False)
+	ax.set_xlabel('Frequency (MHz)')
+
+	if dB:
+		ax.set_ylabel('Relative Power (dB)', x=-10)
+	else:
+		ax.set_ylabel('Relative Power', x=-10)
+
+	ax.set_title('Average RFI Spectrum', y=1.0075)
+
+	ax.annotate('Monitored frequency range: '+str(round(f_lo/1000000,1))+'-'+str(round(np.max(f_total),1))+' MHz ($\\Delta\\nu$ = '+str(round(((np.max(f_total)*1e6-f_lo)/1000000),1))+
+' MHz)\nBandwidth per spectrum: '+str(bandwidth/1000000)+' MHz\nIntegration time per spectrum: '+str(duration)+' sec\nFFT size: '+str(channels), xy=(17, 1290),
+xycoords='axes points', size=32, ha='left', va='top', color='brown')
+
+	ax.grid()
+
+	plt.tight_layout()
+	plt.savefig(plot_file)
+	plt.clf()
+
+def monitor_rfi(f_lo, f_hi, obs_parameters, data='rfi_data'):
+	dev_args = obs_parameters['dev_args']
+	rf_gain = obs_parameters['rf_gain']
+	if_gain = obs_parameters['if_gain']
+	bb_gain = obs_parameters['bb_gain']
+	bandwidth = obs_parameters['bandwidth']
+	channels = obs_parameters['channels']
+	duration = obs_parameters['duration']
+
+	t_sample = 0.1
+
+	# Create RFI data directory
+	if os.path.exists(data):
+		shutil.rmtree(data)
+
+	os.makedirs(data)
+
+	# Iterate over the input frequency range
+	i = 0
+	for frequency in range(int(f_lo), int(f_hi), int(bandwidth)):
+		rfi_parameters = {
+			'dev_args': dev_args,
+			'rf_gain': rf_gain,
+			'if_gain': if_gain,
+			'bb_gain': bb_gain,
+			'frequency': frequency,
+			'bandwidth': bandwidth,
+			'channels': channels,
+			't_sample': t_sample,
+			'duration': duration,
+			'f_lo': f_lo
+		}
+
+		# Run RFI monitor
+		observe(obs_parameters=rfi_parameters, spectrometer='ftf', obs_file=data+'/'+str(i)+'.dat')
+		i += 1
 
 if __name__ == '__main__':
 	# Load argument values
 	parser = argparse.ArgumentParser()
 
 	parser.add_argument('-da', '--dev_args', dest='dev_args',
-                        help='SDR Device Arguments (osmocom Source)', type=str, default='""')
+                        help='SDR Device Arguments (osmocom Source)', type=str, default='')
 	parser.add_argument('-rf', '--rf_gain', dest='rf_gain',
                         help='SDR RF Gain (dB)', type=float, default=10)
 	parser.add_argument('-if', '--if_gain', dest='if_gain',
